/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.external;

import java.util.Objects;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;

/**
 * @author F. Mannhardt
 * 
 */
class XAttributeLiteralExternalImpl extends XAttributeExternalImpl implements XAttributeLiteral {

	private static final long serialVersionUID = 1L;
	
	private final String value;

	public XAttributeLiteralExternalImpl(int key, String value, ExternalStore store) {
		this(key, value, null, store);
	}

	public XAttributeLiteralExternalImpl(int key, String value, XExtension extension, ExternalStore store) {
		super(key, extension, store);
		this.value = value;
	}

	@Override
	public final String getValue() {
		return this.value;
	}

	@Override
	public void setValue(String value) {
		throw new UnsupportedOperationException(IMMUTABLE_ERROR);
	}

	@Override
	public String toString() {
		return this.value;
	}

	@Override
	public final Object clone() {
		// Creating a 'normal' attribute since the clone is not yet saved in MapDB
		XAttributeLiteral clone = new XAttributeLiteralImpl(getKey(), value, getExtension());
		if (hasAttributes()) {
			clone.setAttributes(getAttributes());
		}
		return clone;
	}

	@Override
	public final boolean equals(Object obj) {
		if (obj==this) 
		    return true;
		if (obj instanceof XAttributeLiteral) { // compares types
			XAttributeLiteral other = (XAttributeLiteral) obj;
			return super.equals(other) // compares keys
					&& value.equals(other.getValue()); // compares values
		} else {
			return false;
		}
	}
	
	@Override
	public int hashCode() {
		return Objects.hash(getInternalKey(), value);
	}

	@Override
	public final int compareTo(XAttribute other) {
		if (!(other instanceof XAttributeLiteral)) {
			throw new ClassCastException();
		}
		int result = super.compareTo(other);
		if (result != 0) {
			return result;
		}
		return value.compareTo(((XAttributeLiteral) other).getValue());
	}

}
